@@ -22,22 +22,14 @@
 #include <vte/vte.h>
 #include <adwaita.h>
 
+#include "kgx-livery-manager.h"
+#include "kgx-livery.h"
+#include "kgx-marshals.h"
+#include "kgx-system-info.h"
+#include "kgx-utils.h"
+
 #include "kgx-settings.h"
 
-
-/**
- * DESKTOP_INTERFACE_SETTINGS_SCHEMA:
- * The schema that defines the system fonts
- */
-#define DESKTOP_INTERFACE_SETTINGS_SCHEMA "org.gnome.desktop.interface"
-
-/**
- * MONOSPACE_FONT_KEY_NAME:
- * The name of the key in %DESKTOP_INTERFACE_SETTINGS_SCHEMA for the monospace
- * font
- */
-#define MONOSPACE_FONT_KEY_NAME "monospace-font-name"
-
 #define RESTORE_SIZE_KEY "restore-window-size"
 #define LAST_SIZE_KEY "last-window-size"
 #define LAST_MAXIMISED_KEY "last-window-maximised"
@@ -50,19 +42,27 @@ struct _KgxSettings {
   GObject               parent_instance;
 
   KgxTheme              theme;
+  PangoFontDescription *font;
   double                scale;
   int64_t               scrollback_lines;
   gboolean              audible_bell;
   gboolean              visual_bell;
   gboolean              use_system_font;
   PangoFontDescription *custom_font;
+  int64_t               scrollback_limit;
+  gboolean              ignore_scrollback_limit;
+  gboolean              software_flow_control;
+  KgxLivery            *livery;
+  gboolean              transparency;
+
+  KgxLiveryManager     *livery_manager;
 
   GSettings            *settings;
-  GSettings            *desktop_interface;
+  KgxSystemInfo        *system_info;
 };
 
 
-G_DEFINE_TYPE (KgxSettings, kgx_settings, G_TYPE_OBJECT)
+G_DEFINE_FINAL_TYPE (KgxSettings, kgx_settings, G_TYPE_OBJECT)
 
 
 enum {
@@ -78,9 +78,13 @@ enum {
   PROP_VISUAL_BELL,
   PROP_USE_SYSTEM_FONT,
   PROP_CUSTOM_FONT,
+  PROP_SCROLLBACK_LIMIT,
+  PROP_IGNORE_SCROLLBACK_LIMIT,
+  PROP_SOFTWARE_FLOW_CONTROL,
+  PROP_LIVERY,
+  PROP_TRANSPARENCY,
   LAST_PROP
 };
-
 static GParamSpec *pspecs[LAST_PROP] = { NULL, };
 
 
@@ -89,8 +93,16 @@ kgx_settings_dispose (GObject *object)
 {
   KgxSettings *self = KGX_SETTINGS (object);
 
+  g_clear_pointer (&self->font, pango_font_description_free);
+  g_clear_pointer (&self->custom_font, pango_font_description_free);
+
   g_clear_object (&self->settings);
-  g_clear_object (&self->desktop_interface);
+  g_clear_object (&self->system_info);
+
+  g_clear_pointer (&self->custom_font, pango_font_description_free);
+  g_clear_pointer (&self->livery, kgx_livery_unref);
+
+  g_clear_object (&self->livery_manager);
 
   G_OBJECT_CLASS (kgx_settings_parent_class)->dispose (object);
 }
@@ -125,25 +137,66 @@ kgx_settings_set_property (GObject      *object,
     case PROP_THEME:
       self->theme = g_value_get_enum (value);
       g_object_notify_by_pspec (object, pspecs[PROP_RESOLVED_THEME]);
+      g_object_notify_by_pspec (object, pspecs[PROP_LIVERY]);
+      break;
+    case PROP_FONT:
+      g_clear_pointer (&self->font, pango_font_description_free);
+      self->font = g_value_dup_boxed (value);
+      g_object_notify_by_pspec (object, pspecs[PROP_FONT]);
       break;
     case PROP_FONT_SCALE:
       update_scale (self, g_value_get_double (value));
       break;
     case PROP_SCROLLBACK_LINES:
-      kgx_settings_set_scrollback (self, g_value_get_int64 (value));
+      kgx_set_int64_prop (object, pspec, &self->scrollback_lines, value);
       break;
     case PROP_AUDIBLE_BELL:
-      kgx_settings_set_audible_bell (self, g_value_get_boolean (value));
+      kgx_set_boolean_prop (object,
+                            pspec,
+                            &self->audible_bell,
+                            value);
       break;
     case PROP_VISUAL_BELL:
-      kgx_settings_set_visual_bell (self, g_value_get_boolean (value));
+      kgx_set_boolean_prop (object,
+                            pspec,
+                            &self->visual_bell,
+                            value);
       break;
     case PROP_USE_SYSTEM_FONT:
-      kgx_settings_set_use_system_font (self, g_value_get_boolean (value));
+      kgx_set_boolean_prop (object,
+                            pspec,
+                            &self->use_system_font,
+                            value);
       break;
     case PROP_CUSTOM_FONT:
       kgx_settings_set_custom_font (self, g_value_get_boxed (value));
       break;
+    case PROP_SCROLLBACK_LIMIT:
+      kgx_set_int64_prop (object, pspec, &self->scrollback_limit, value);
+      break;
+    case PROP_IGNORE_SCROLLBACK_LIMIT:
+      kgx_set_boolean_prop (object,
+                            pspec,
+                            &self->ignore_scrollback_limit,
+                            value);
+      break;
+    case PROP_SOFTWARE_FLOW_CONTROL:
+      kgx_set_boolean_prop (object,
+                            pspec,
+                            &self->software_flow_control,
+                            value);
+      break;
+    case PROP_TRANSPARENCY:
+      kgx_set_boolean_prop (object,
+                            pspec,
+                            &self->transparency,
+                            value);
+      break;
+    case PROP_LIVERY:
+      if (kgx_set_livery (&self->livery, g_value_get_boxed (value))) {
+        g_object_notify_by_pspec (object, pspecs[PROP_LIVERY]);
+      }
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -167,7 +220,7 @@ kgx_settings_get_property (GObject    *object,
       g_value_set_enum (value, kgx_settings_get_resolved_theme (self));
       break;
     case PROP_FONT:
-      g_value_take_boxed (value, kgx_settings_get_font (self));
+      g_value_set_boxed (value, self->font);
       break;
     case PROP_FONT_SCALE:
       g_value_set_double (value, self->scale);
@@ -193,6 +246,21 @@ kgx_settings_get_property (GObject    *object,
     case PROP_CUSTOM_FONT:
       g_value_set_boxed (value, self->custom_font);
       break;
+    case PROP_SCROLLBACK_LIMIT:
+      g_value_set_int64 (value, self->scrollback_limit);
+      break;
+    case PROP_IGNORE_SCROLLBACK_LIMIT:
+      g_value_set_boolean (value, self->ignore_scrollback_limit);
+      break;
+    case PROP_SOFTWARE_FLOW_CONTROL:
+      g_value_set_boolean (value, self->software_flow_control);
+      break;
+    case PROP_LIVERY:
+      g_value_set_boxed (value, kgx_settings_get_livery (self));
+      break;
+    case PROP_TRANSPARENCY:
+      g_value_set_boolean (value, self->transparency);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -214,8 +282,6 @@ kgx_settings_class_init (KgxSettingsClass *klass)
    *
    * The palette to use, one of the values of #KgxTheme
    *
-   * Officially only "night" exists, "hacker" is just a little fun
-   *
    * Bound to ‘theme’ GSetting so changes persist
    */
   pspecs[PROP_THEME] =
@@ -231,7 +297,7 @@ kgx_settings_class_init (KgxSettingsClass *klass)
   pspecs[PROP_FONT] =
     g_param_spec_boxed ("font", NULL, NULL,
                         PANGO_TYPE_FONT_DESCRIPTION,
-                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
+                        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
   pspecs[PROP_FONT_SCALE] =
     g_param_spec_double ("font-scale", NULL, NULL,
@@ -282,18 +348,93 @@ kgx_settings_class_init (KgxSettingsClass *klass)
                         PANGO_TYPE_FONT_DESCRIPTION,
                         G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS | G_PARAM_EXPLICIT_NOTIFY);
 
+  pspecs[PROP_SCROLLBACK_LIMIT] =
+    g_param_spec_int64 ("scrollback-limit", NULL, NULL,
+                        G_MININT64, G_MAXINT64, 10000,
+                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  pspecs[PROP_IGNORE_SCROLLBACK_LIMIT] =
+    g_param_spec_boolean ("ignore-scrollback-limit", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  pspecs[PROP_SOFTWARE_FLOW_CONTROL] =
+    g_param_spec_boolean ("software-flow-control", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  pspecs[PROP_LIVERY] =
+    g_param_spec_boxed ("livery", NULL, NULL,
+                        KGX_TYPE_LIVERY,
+                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
+  pspecs[PROP_TRANSPARENCY] =
+    g_param_spec_boolean ("transparency", NULL, NULL,
+                          FALSE,
+                          G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
+
   g_object_class_install_properties (object_class, LAST_PROP, pspecs);
 }
 
 
-static void
-system_font_changed (GSettings    *settings,
-                     const char   *key,
-                     KgxSettings  *self)
+static gboolean
+variant_to_value (GValue *value, GVariant *variant, gpointer data)
 {
-  if (self->use_system_font) {
-    g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_FONT]);
+  g_value_set_variant (value, variant);
+
+  return TRUE;
+}
+
+
+static GVariant *
+value_to_variant (const GValue       *value,
+                  const GVariantType *variant_ty,
+                  gpointer            data)
+{
+  return g_value_dup_variant (value);
+}
+
+
+static gboolean
+resolve_livery (GValue   *value,
+                GVariant *variant,
+                gpointer  data)
+{
+  KgxSettings *self = data;
+  const char *uuid = g_variant_get_string (variant, NULL);
+  KgxLivery *livery = kgx_livery_manager_resolve (self->livery_manager, uuid);
+
+  if (!livery) {
+    g_log_structured (G_LOG_DOMAIN,
+                      G_LOG_LEVEL_WARNING,
+                      "CODE_FILE", __FILE__,
+                      "CODE_LINE", G_STRINGIFY (__LINE__),
+                      "CODE_FUNC", G_STRFUNC,
+                      "KGX_LIVERY_UUID", uuid,
+                      "MESSAGE", "settings: ‘%s’ is unknown, using default livery",
+                      uuid);
+
+    livery = kgx_livery_manager_dup_fallback (self->livery_manager);
   }
+
+  g_value_set_boxed (value, livery);
+
+  return TRUE;
+}
+
+
+static GVariant *
+select_livery (const GValue       *value,
+               const GVariantType *variant_ty,
+               gpointer            data)
+{
+  KgxLivery *livery = g_value_get_boxed (value);
+
+  if (livery) {
+    return NULL;
+  }
+
+  return g_variant_new_string (kgx_livery_get_uuid (livery));
 }
 
 
@@ -350,17 +491,78 @@ encode_font (const GValue       *value,
   return g_variant_new_take_string (pango_font_description_to_string (font));
 }
 
+
 static void
 dark_changed (KgxSettings *self)
 {
   if (self->theme == KGX_THEME_AUTO) {
     g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_RESOLVED_THEME]);
+    g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_LIVERY]);
   }
 }
 
+
+static PangoFontDescription *
+resolve_font (GObject              *object,
+              gboolean              use_system,
+              PangoFontDescription *system,
+              PangoFontDescription *custom)
+{
+  return pango_font_description_copy (use_system ? system : custom);
+}
+
+
+static inline void
+setup_font_expression (KgxSettings *self)
+{
+  GtkExpression *params[] = {
+    gtk_property_expression_new (KGX_TYPE_SETTINGS, NULL, "use-system-font"),
+    gtk_property_expression_new (KGX_TYPE_SYSTEM_INFO,
+                                 gtk_constant_expression_new (KGX_TYPE_SYSTEM_INFO, self->system_info),
+                                 "monospace-font"),
+    gtk_property_expression_new (KGX_TYPE_SETTINGS, NULL, "custom-font"),
+  };
+  GtkExpression *exp =
+    gtk_cclosure_expression_new (PANGO_TYPE_FONT_DESCRIPTION,
+                                 kgx_marshals_BOXED__BOOLEAN_BOXED_BOXED,
+                                 G_N_ELEMENTS (params), params,
+                                 G_CALLBACK (resolve_font), NULL, NULL);
+
+  gtk_expression_bind (exp, self, "font", self);
+}
+
+
+static int64_t
+resolve_lines (GObject              *object,
+               gboolean              ignore_limit,
+               int64_t               limit)
+{
+  return ignore_limit ? -1 : limit;
+}
+
+
+static inline void
+setup_lines_expression (KgxSettings *self)
+{
+  GtkExpression *params[] = {
+    gtk_property_expression_new (KGX_TYPE_SETTINGS, NULL, "ignore-scrollback-limit"),
+    gtk_property_expression_new (KGX_TYPE_SETTINGS, NULL, "scrollback-limit"),
+  };
+  GtkExpression *exp =
+    gtk_cclosure_expression_new (G_TYPE_INT64,
+                                 kgx_marshals_INT64__BOOLEAN_INT64,
+                                 G_N_ELEMENTS (params), params,
+                                 G_CALLBACK (resolve_lines), NULL, NULL);
+
+  gtk_expression_bind (exp, self, "scrollback-lines", self);
+}
+
+
 static void
 kgx_settings_init (KgxSettings *self)
 {
+  self->livery_manager = g_object_new (KGX_TYPE_LIVERY_MANAGER, NULL);
+
   self->settings = g_settings_new (KGX_APPLICATION_ID);
   g_settings_bind (self->settings, "theme",
                    self, "theme",
@@ -368,9 +570,6 @@ kgx_settings_init (KgxSettings *self)
   g_settings_bind (self->settings, "font-scale",
                    self, "font-scale",
                    G_SETTINGS_BIND_DEFAULT);
-  g_settings_bind (self->settings, "scrollback-lines",
-                   self, "scrollback-lines",
-                   G_SETTINGS_BIND_DEFAULT);
   g_settings_bind (self->settings, "audible-bell",
                    self, "audible-bell",
                    G_SETTINGS_BIND_DEFAULT);
@@ -386,17 +585,39 @@ kgx_settings_init (KgxSettings *self)
                                 decode_font,
                                 encode_font,
                                 NULL, NULL);
+  g_settings_bind (self->settings, "scrollback-lines",
+                   /* Yes, this is intentional */
+                   self, "scrollback-limit",
+                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (self->settings, "ignore-scrollback-limit",
+                   self, "ignore-scrollback-limit",
+                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (self->settings, "software-flow-control",
+                   self, "software-flow-control",
+                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind (self->settings, "transparency",
+                   self, "transparency",
+                   G_SETTINGS_BIND_DEFAULT);
+  g_settings_bind_with_mapping (self->settings, "custom-liveries",
+                                self->livery_manager, "custom-liveries",
+                                G_SETTINGS_BIND_DEFAULT, variant_to_value,
+                                value_to_variant,
+                                NULL, NULL);
+  g_settings_bind_with_mapping (self->settings, "livery",
+                                self, "livery",
+                                G_SETTINGS_BIND_DEFAULT,
+                                resolve_livery,
+                                select_livery,
+                                self, NULL);
 
   g_signal_connect (self->settings,
-                    "changed::restore-window-size",
+                    "changed::" RESTORE_SIZE_KEY,
                     G_CALLBACK (restore_window_size_changed),
                     self);
 
-  self->desktop_interface = g_settings_new (DESKTOP_INTERFACE_SETTINGS_SCHEMA);
-  g_signal_connect (self->desktop_interface,
-                    "changed::" MONOSPACE_FONT_KEY_NAME,
-                    G_CALLBACK (system_font_changed),
-                    self);
+  self->system_info = g_object_new (KGX_TYPE_SYSTEM_INFO, NULL);
+  setup_font_expression (self);
+  setup_lines_expression (self);
 
   g_signal_connect_object (adw_style_manager_get_default (),
                            "notify::dark", G_CALLBACK (dark_changed),
@@ -428,33 +649,6 @@ kgx_settings_get_resolved_theme (KgxSettings *self)
   return KGX_THEME_DAY;
 }
 
-/**
- * kgx_settings_get_font:
- * @self: the #KgxSettings
- *
- * Creates a #PangoFontDescription for the system monospace font when
- * #KgxSettings:use-system-font or the user defined font specified by
- * #KgxSettings:custom-font
- *
- * Returns: (transfer full): a new #PangoFontDescription
- */
-PangoFontDescription *
-kgx_settings_get_font (KgxSettings *self)
-{
-  g_autofree char *font = NULL;
-
-  g_return_val_if_fail (KGX_IS_SETTINGS (self), NULL);
-
-  if (!self->use_system_font && self->custom_font) {
-    return pango_font_description_copy (self->custom_font);
-  }
-
-  font = g_settings_get_string (self->desktop_interface,
-                                MONOSPACE_FONT_KEY_NAME);
-
-  return pango_font_description_from_string (font);
-}
-
 
 void
 kgx_settings_increase_scale (KgxSettings *self)
@@ -532,8 +726,8 @@ kgx_settings_set_custom_shell (KgxSettings *self, const char *const *shell)
 
 
 void
-kgx_settings_set_scrollback (KgxSettings *self,
-                             int64_t      value)
+kgx_settings_set_scrollback_limit (KgxSettings *self,
+                                   int64_t      value)
 {
   g_return_if_fail (KGX_IS_SETTINGS (self));
 
@@ -543,7 +737,7 @@ kgx_settings_set_scrollback (KgxSettings *self,
 
   self->scrollback_lines = value;
 
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_SCROLLBACK_LINES]);
+  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_SCROLLBACK_LIMIT]);
 }
 
 
@@ -605,21 +799,6 @@ kgx_settings_get_audible_bell (KgxSettings *self)
 }
 
 
-void
-kgx_settings_set_audible_bell (KgxSettings *self,
-                               gboolean     audible_bell)
-{
-  g_return_if_fail (KGX_IS_SETTINGS (self));
-
-  if (self->audible_bell == audible_bell)
-    return;
-
-  self->audible_bell = audible_bell;
-
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_AUDIBLE_BELL]);
-}
-
-
 gboolean
 kgx_settings_get_visual_bell (KgxSettings *self)
 {
@@ -629,46 +808,6 @@ kgx_settings_get_visual_bell (KgxSettings *self)
 }
 
 
-void
-kgx_settings_set_visual_bell (KgxSettings *self,
-                              gboolean     visual_bell)
-{
-  g_return_if_fail (KGX_IS_SETTINGS (self));
-
-  if (self->visual_bell == visual_bell)
-    return;
-
-  self->visual_bell = visual_bell;
-
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_VISUAL_BELL]);
-}
-
-
-gboolean
-kgx_settings_get_use_system_font (KgxSettings *self)
-{
-  g_return_val_if_fail (KGX_IS_SETTINGS (self), FALSE);
-
-  return self->use_system_font;
-}
-
-
-void
-kgx_settings_set_use_system_font (KgxSettings *self,
-                                  gboolean     use_system_font)
-{
-  g_return_if_fail (KGX_IS_SETTINGS (self));
-
-  if (self->use_system_font == use_system_font)
-    return;
-
-  self->use_system_font = use_system_font;
-
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_USE_SYSTEM_FONT]);
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_FONT]);
-}
-
-
 /**
  * kgx_settings_get_custom_font:
  *
@@ -699,5 +838,34 @@ kgx_settings_set_custom_font (KgxSettings          *self,
   self->custom_font = pango_font_description_copy (custom_font);
 
   g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_CUSTOM_FONT]);
-  g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_FONT]);
+}
+
+
+gboolean
+kgx_settings_get_software_flow_control (KgxSettings *self)
+{
+  g_return_val_if_fail (KGX_IS_SETTINGS (self), FALSE);
+
+  return self->software_flow_control;
+}
+
+
+KgxLivery *
+kgx_settings_get_livery (KgxSettings *self)
+{
+  g_return_val_if_fail (KGX_IS_SETTINGS (self), NULL);
+
+  return self->livery;
+}
+
+
+void
+kgx_settings_set_livery (KgxSettings *restrict self,
+                         KgxLivery   *restrict livery)
+{
+  g_return_if_fail (KGX_IS_SETTINGS (self));
+
+  if (kgx_set_livery (&self->livery, livery)) {
+    g_object_notify_by_pspec (G_OBJECT (self), pspecs[PROP_LIVERY]);
+  }
 }
